# ORM {#top}

Kohana 3.0 включает мощный модуль ORM, который использует паттерн Active Record и автоопределение информации о списке полей БД модели.

Модуль ORM включен в дистрибутив Kohana 3.0, но нуждается в подключении перед его использованием. В файле `application/bootstrap.php` отредактируйте вызов [Kohana::modules] и добавьте модуль ORM:

	Kohana::modules(array(
		...
		'orm' => MODPATH.'orm',
		...
	));

## Настройка {#configuration}

ORM требует небольшую настройку перед использованием.  Наследуйте Вашу модуль от ORM:

	class Model_User extends ORM
	{
		...
	}

В примере выше, модель будет искать таблицу `users` в БД по умолчанию.

### Свойства модели, отвечающие за конфигурацию

Следующие свойства используются для настройки каждой модели:

Тип       | Название        |  Описание                      | Значение по умолчанию
----------|-----------------|--------------------------------| -------------------------
`string`  |  _table_name    | Используемая таблица БД        |
`string`  | _db             | Название БД                    |`default`
`string`  | _primary_key    | Поле - первичный ключ          |`id`
`string`  | _primary_val    | Титульное поле                 |`name`

## Использование ORM

### Загрузка записи

Для создания экземпляра модели используйте метод [ORM::factory] или конструктор [ORM::__construct]:

	$user = ORM::factory('user');
	// или
	$user = new Model_User();

Конструктор и фабричный метод также поддерживают значение первичного ключа для загрузки конкретной записи:

	// Загружаем пользователя с ID 5
	$user = ORM::factory('user', 5);

[ORM::loaded] позволяет убедиться, что модель была загружена успешно.

### Поиск записи

ORM поддерживает большинство методов  [Database] для полноценного поиска данных модели.  В свойстве `_db_methods` перечислен полный список поддерживаемых методов.  Записи извлекаются после вызовов [ORM::find] или [ORM::find_all].

	// Извлекаем первого активного пользователя по имени Bob
	$user = ORM::factory('user')
		->where('active', '=', TRUE)
		->where('name', '=', 'Bob')
		->find();

	// Ищем всех активных пользователей по имени Bob
	$users = ORM::factory('user')
		...
		->find_all();
	
Когда Вы запрашиваете список моделей через [ORM::find_all], перебирать его можно аналогично обычным выборкам из БД:

	foreach ($users as $user)
	{
		...
	}

### Доступ к свойствам модели

Все свойства модели доступны через "магические" методы `__get` и `__set`.

	$user = ORM::factory('user', 5);
	
	// Выводит имя пользователя
	echo $user->name;

	// Изменяет имя пользователя
	$user->name = 'Bob';

Для хранения данных/свойств, которые отсутствуют в таблице модели, надо использовать атрибут `_ignored_columns`.

	class Model_User extends ORM
	{
		...
		protected $_ignored_columns = array('field1', 'field2', ...)
		...
	}

### Создаем и сохраняем записи

Метод [ORM::save] используется как для создания новых записей, так и для обновления существующих.

	// Создаем запись
	$user = ORM::factory('user');
	$user->name = 'New user';
	$user->save();

	// Редактируем запись
	$user = ORM::factory('user', 5);
	$user->name = 'User 2';
	$user->save();

Вы можете обновить множество записей с помощью методоа [ORM::save_all]:

	$user = ORM::factory('user');
	$user->name = 'Bob';

	// Все активные пользователи получат имя 'Bob'
	$user->where('active', '=', TRUE)->save_all();

[ORM::saved] проверяет, сохранена ли модель.

### Удаление записей

Записи удаляются методами [ORM::delete] и [ORM::delete_all].  Эти методы работают аналогично описанному выше сохранению, за исключением того, что [ORM::delete] принимает необязательный параметр `id` для удаляемой записи.